package version7_2_2_1.display;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import version7_2_2_1.Interface.InteractionList;
import version7_2_2_1.platform.Agent;

/**
 * Panel to display the links between positions
 * @author simon
 */

/* - inherited from EnvPanel :
 *   Agent agent      : pointer to the agent
 */
public class MemoryPanel extends EnvPanel implements MouseListener{

	private static final long serialVersionUID = 1L;
	
	private boolean measure=false;
	
	private int clic_x=0;
	private int clic_y=0;
	
	private boolean[] display;
	private static int gap=8;			// distance between nodes
	private int color=0;				// color of displayed interactions (0-> green, 1 -> blue, 2 -> red)
	
	private float[][][] grid;
	
	public MemoryPanel(Agent a){
		super(a);
		addMouseListener(this);

		display=new boolean[6];
		for (int i=0;i<6;i++) display[i]=false;
		
		grid =new float[InteractionList.size1][InteractionList.size2][4];
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2;j++){
				grid[i][j][0]=i*2.9f;
				grid[i][j][1]=j*2.9f;
				
				grid[i][j][2]=-1;
				grid[i][j][3]=-1;
			}
		}
		
	}
	
	public void setAgent(Agent a){
		agent=a;
	}
	
	public void paintComponent(Graphics g){
		g.setColor(Color.white);
		
		g.fillRect(0, 0, 1200, 1000);
		
		// draw mesh type switch
		if (agent.nodeList.getMesh()) g.setColor(Color.blue);
		else g.setColor(Color.red);
		g.fillRect(20, 70, 10, 10);
		
		// measure switch
		if (measure) g.setColor(Color.red);
		else g.setColor(Color.black);
		g.fillRect(40, 70, 10, 10);
		
		// draw color switch
		g.setColor(Color.green);
		g.fillRect(20,100, 10, 10);
		g.setColor(Color.blue);
		g.fillRect(20,120, 10, 10);
		g.setColor(Color.red);
		g.fillRect(20,140, 10, 10);
		
		g.setColor(Color.black);
		g.drawRect(19, 99+20*color, 10, 10);
		
		// draw interaction switch
		for (int i=0;i<6;i++){
			if (display[i]) g.setColor(Color.red);
			else g.setColor(Color.black);
			g.fillRect(20, 180+20*i, 10, 10);
		}
		
		int nbPos=InteractionList.size1*InteractionList.size2*3;
		
		// draw nodes		
		for (int i=0; i<nbPos;i++){
			g.setColor(Color.black);
			g.fillOval(150+(int)(agent.nodeList.get(i).x*gap)-1, 
					    50+(int)(agent.nodeList.get(i).y*gap)-1, 2, 2);
		}
		
		for (int l=0;l<6;l++){	
			if (display[l]){
				for (int i=color; i<nbPos;i+=3){

					if (i%3==0) g.setColor(Color.green);
					if (i%3==1) g.setColor(Color.blue);
					if (i%3==2) g.setColor(Color.red);
					
					g.fillOval(150+(int)(agent.nodeList.get(i+l*nbPos).x*gap)-3,
							    50+(int)(agent.nodeList.get(i+l*nbPos).y*gap)-3, 5, 5);
				}
			}
		}
		
		g.setColor(Color.black);
		for (int i=0; i<7;i++){
			g.fillOval(150+(int)(agent.nodeList.get(i+InteractionList.nbDF).x*gap)-3,
					    50+(int)(agent.nodeList.get(i+InteractionList.nbDF).y*gap)-3, 5, 5);
		}
		
		g.setColor(Color.black);
		
		if (!agent.nodeList.getMesh()){
			// draw links between each interaction and the barycenter of its signature
			for (int l=0;l<6;l++){

				if (display[l]){
					
					for (int i=color; i<nbPos;i+=3){
	
						// for each node, draw links between this node and average position of high weights
						float x=0;
						float y=0;
						float n=0;
						
						for (int k=0;k<6;k++){
							for (int j=1; j<nbPos;j++){
								if (agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos]>6){	
									x+=agent.nodeList.get(j).x * agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
									y+=agent.nodeList.get(j).y * agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
									n+=agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
								}
							}
						}
						
						if (n>0){
							x=x/n;
							y=y/n;
							g.drawLine(150+(int)(agent.nodeList.get(i).x*gap),
								   	    50+(int)(agent.nodeList.get(i).y*gap),
								   	   150+(int)(x*gap),
								   	    50+(int)(y*gap));
							
							grid[((i-color)/3)/InteractionList.size2][((i-color)/3)%InteractionList.size2][2]=x;
							grid[((i-color)/3)/InteractionList.size2][((i-color)/3)%InteractionList.size2][3]=y;
						}
						else{
							grid[((i-color)/3)/InteractionList.size2][((i-color)/3)%InteractionList.size2][2]=-1;
							grid[((i-color)/3)/InteractionList.size2][((i-color)/3)%InteractionList.size2][3]=-1;
						}
					}
				}
			}
		}
		else{
			// draw mesh
			g.setColor(Color.black);
			for (int l=0;l<6;l++){
				if (display[l]){
					for (int i=0;i<InteractionList.size1;i++){
						for (int j=0;j<InteractionList.size2;j++){
							
							for (int c=color;c<color+1;c+=3){
								
							// connect each node of a row with the next node
								if (i<InteractionList.size1-1){
									if ((agent.nodeList.get(l*nbPos+3*j+InteractionList.size2*3* i   +c).x*gap<730 || agent.nodeList.get(l*nbPos+3*j+InteractionList.size2*3* i   +c).y*gap<420)
									&&  (agent.nodeList.get(l*nbPos+3*j+InteractionList.size2*3*(i+1)+c).x*gap<730 || agent.nodeList.get(l*nbPos+3*j+InteractionList.size2*3*(i+1)+c).y*gap<420)){
										g.drawLine(150+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*i +c).x*gap),
											   	    50+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*i +c).y*gap),
											   	   150+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*(i+1) +c).x*gap),
											   	    50+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*(i+1) +c).y*gap));
									}
								}
								
								// and each node of a column with the next
								if (j<InteractionList.size2-1){
									if ((agent.nodeList.get(l*nbPos+3* j   +InteractionList.size2*3*i+c).x*gap<730 || agent.nodeList.get(l*nbPos+3* j   +InteractionList.size2*3*i+c).y*gap<420)
									&&  (agent.nodeList.get(l*nbPos+3*(j+1)+InteractionList.size2*3*i+c).x*gap<730 || agent.nodeList.get(l*nbPos+3*(j+1)+InteractionList.size2*3*i+c).y*gap<420)){
										g.drawLine(150+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*i +c).x*gap),
											   	    50+(int)(agent.nodeList.get(l*nbPos + 3*j + InteractionList.size2*3*i +c).y*gap),
											   	   150+(int)(agent.nodeList.get(l*nbPos + 3*(j+1) + InteractionList.size2*3*i +c).x*gap),
											   	    50+(int)(agent.nodeList.get(l*nbPos + 3*(j+1) + InteractionList.size2*3*i +c).y*gap));
									}
								}
							}
						}
					}
				}
			}
		}
		
		
		
		// grid simulation
		if (measure){		
			for (int k=0;k<1000;k++){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2-1;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i][j+1][0])* (grid[i][j][0]-grid[i][j+1][0]) 
								  				 + (grid[i][j][1]-grid[i][j+1][1])* (grid[i][j][1]-grid[i][j+1][1]) );
						if (d!=0){
							
							if (d>0){
								grid[i][j][0]+= 0.05 * (grid[i][j+1][0]-grid[i][j][0]) * (d-3) /d ;
								grid[i][j][1]+= 0.05 * (grid[i][j+1][1]-grid[i][j][1]) * (d-3) /d;
								
								grid[i][j+1][0]-= 0.05 * (grid[i][j+1][0]-grid[i][j][0]) * (d-3) /d;
								grid[i][j+1][1]-= 0.05 * (grid[i][j+1][1]-grid[i][j][1]) * (d-3) /d;
							}
						}
						else{
							grid[i][j][0]+=0.1;
							grid[i][j][1]+=0.1;
						}
					}
				}
				
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2-2;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i][j+2][0])* (grid[i][j][0]-grid[i][j+2][0]) 
								  				 + (grid[i][j][1]-grid[i][j+2][1])* (grid[i][j][1]-grid[i][j+2][1]) );
						
						if (d!=0){
							
							if (d>0){
								grid[i][j][0]+= 0.02 * (grid[i][j+2][0]-grid[i][j][0]) * (d-6) /d ;
								grid[i][j][1]+= 0.02 * (grid[i][j+2][1]-grid[i][j][1]) * (d-6) /d;
								
								grid[i][j+2][0]-= 0.02 * (grid[i][j+2][0]-grid[i][j][0]) * (d-6) /d;
								grid[i][j+2][1]-= 0.02 * (grid[i][j+2][1]-grid[i][j][1]) * (d-6) /d;
							}
						}
					}
				}
				
				for (int i=0;i<InteractionList.size1-1;i++){
					for (int j=0;j<InteractionList.size2;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i+1][j][0])* (grid[i][j][0]-grid[i+1][j][0]) 
								  				 + (grid[i][j][1]-grid[i+1][j][1])* (grid[i][j][1]-grid[i+1][j][1]) );
						
						if (d!=0){
							
							grid[i][j][0]+= 0.05 * (grid[i+1][j][0]-grid[i][j][0]) * (d-3) /d;
							grid[i][j][1]+= 0.05 * (grid[i+1][j][1]-grid[i][j][1]) * (d-3) /d;
							
							grid[i+1][j][0]-= 0.05 * (grid[i+1][j][0]-grid[i][j][0]) * (d-3) /d;
							grid[i+1][j][1]-= 0.05 * (grid[i+1][j][1]-grid[i][j][1]) * (d-3) /d;
						}
						else{
							grid[i][j][0]+=0.1;
							grid[i][j][1]+=0.1;
						}
					}
				}
				
				for (int i=0;i<InteractionList.size1-2;i++){
					for (int j=0;j<InteractionList.size2;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i+2][j][0])* (grid[i][j][0]-grid[i+2][j][0]) 
								  				 + (grid[i][j][1]-grid[i+2][j][1])* (grid[i][j][1]-grid[i+2][j][1]) );
						
						if (d!=0){
							
							grid[i][j][0]+= 0.02 * (grid[i+2][j][0]-grid[i][j][0]) * (d-6) /d;
							grid[i][j][1]+= 0.02 * (grid[i+2][j][1]-grid[i][j][1]) * (d-6) /d;
							
							grid[i+2][j][0]-= 0.02 * (grid[i+2][j][0]-grid[i][j][0]) * (d-6) /d;
							grid[i+2][j][1]-= 0.02 * (grid[i+2][j][1]-grid[i][j][1]) * (d-6) /d;
						}
					}
				}
				
				for (int i=0;i<InteractionList.size1-1;i++){
					for (int j=0;j<InteractionList.size2-1;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i+1][j+1][0])* (grid[i][j][0]-grid[i+1][j+1][0]) 
								  				 + (grid[i][j][1]-grid[i+1][j+1][1])* (grid[i][j][1]-grid[i+1][j+1][1]) );
						
						if (d!=0){
							
							grid[i][j][0]+= 0.02 * (grid[i+1][j+1][0]-grid[i][j][0]) * (d-Math.sqrt(18)) /d;
							grid[i][j][1]+= 0.02 * (grid[i+1][j+1][1]-grid[i][j][1]) * (d-Math.sqrt(18)) /d;
							
							grid[i+1][j+1][0]-= 0.02 * (grid[i+1][j+1][0]-grid[i][j][0]) * (d-Math.sqrt(18)) /d;
							grid[i+1][j+1][1]-= 0.02 * (grid[i+1][j+1][1]-grid[i][j][1]) * (d-Math.sqrt(18)) /d;
						}
					}
				}
				
				for (int i=1;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2-1;j++){
						
						float d=(float)Math.sqrt( (grid[i][j][0]-grid[i-1][j+1][0])* (grid[i][j][0]-grid[i-1][j+1][0]) 
								  				 + (grid[i][j][1]-grid[i-1][j+1][1])* (grid[i][j][1]-grid[i-1][j+1][1]) );
						
						if (d!=0){
							
							grid[i][j][0]+= 0.02 * (grid[i-1][j+1][0]-grid[i][j][0]) * (d-Math.sqrt(18)) /d;
							grid[i][j][1]+= 0.02 * (grid[i-1][j+1][1]-grid[i][j][1]) * (d-Math.sqrt(18)) /d;
							
							grid[i-1][j+1][0]-= 0.02 * (grid[i-1][j+1][0]-grid[i][j][0]) * (d-Math.sqrt(18)) /d;
							grid[i-1][j+1][1]-= 0.02 * (grid[i-1][j+1][1]-grid[i][j][1]) * (d-Math.sqrt(18)) /d;
						}
					}
				}
				
				/////////////////////////////////////////////
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2;j++){
						
						if (grid[i][j][2]!=-1 && grid[i][j][3]!=-1){
	
							grid[i][j][0]-= 0.0001 * (grid[i][j][0]-grid[i][j][2]);
							grid[i][j][1]-= 0.0001 * (grid[i][j][1]-grid[i][j][3]);
							
						}
					}
				}
				
			}
			
			g.setColor(Color.red);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2;j++){
					g.drawOval(150+(int)(grid[i][j][2]*gap)-2, 50+(int)(grid[i][j][3]*gap)-2, 5, 5);
				}
			}
			g.setColor(Color.cyan);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2;j++){
					g.drawOval(150+(int)(grid[i][j][0]*gap)-2, 50+(int)(grid[i][j][1]*gap)-2, 5, 5);
				}
			}
			
			float m_x1=((grid[14][0][0]+grid[15][0][0])/2 /3) -14.5f;
			float m_y1=(grid[14][0][1]+grid[15][0][1])/2 /3;
			
			float m_x2=((grid[14][14][0]+grid[15][14][0])/2 /3) -14.5f;
			float m_y2=(grid[14][14][1]+grid[15][14][1])/2 /3;
			
			float h=(float)(Math.sqrt( (m_x1-m_x2)*(m_x1-m_x2) + (m_y1-m_y2)*(m_y1-m_y2)));
			
			float theta=0;
			if (h!=0) theta=(float)(180*Math.asin((m_x1-m_x2)/h)/Math.PI);
			
			float error=0;
			float nb=0;
			
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2;j++){
					
					if (grid[i][j][2]!=-1 && grid[i][j][3]!=-1){
						
						error+=Math.sqrt( (grid[i][j][0]-grid[i][j][2])* (grid[i][j][0]-grid[i][j][2]) 
										+ (grid[i][j][1]-grid[i][j][3])* (grid[i][j][1]-grid[i][j][3]) );	
						nb++;
					}
				}
			}
			if (nb>0) error=error/nb/3;
			System.out.println(" +++ "+m_x1+", "+m_y1+" ; "+theta+" ; average error : "+error);
		}
	
	}
	
	
	///////////////////////////////////////////////////////////////////////////
	public void mouseClicked(MouseEvent e) {

		clic_x=e.getX();
		clic_y=e.getY();
		
		float dmin=1000;
		float d=10;
		
		boolean found=false;
		
		
		if (clic_x>=20 && clic_x<30){
			
			// mesh type switch
			if (clic_y>=70 && clic_y<80){
				found=true;
				agent.nodeList.changeMesh();
			}
			
			// color switch
			for (int c=0;c<3;c++){
				if (clic_y>=100+c*20 && clic_y<110+c*20){
					found=true;
					color=c;
				}
			}
			
			// interaction switch
			for (int i=0;i<6;i++){
				if (clic_y>=180+i*20 && clic_y<190+i*20){
					found=true;
					display[i]=!display[i];
				}
			}
		}
		
		if (clic_x>=40 && clic_x<50){
			// measure switch
			if (clic_y>=70 && clic_y<80){
				found=true;
				measure =!measure;
			}
		}
		
		if (!found){
			agent.observer.setSelectedNode(-1);
			
			// select a node
			for (int i=0;i<agent.nodeList.size();i++){
				if (agent.nodeList.get(i).color==this.color){
					d=(clic_x-(150+agent.nodeList.get(i).x*8-3)) * (clic_x-(150+agent.nodeList.get(i).x*8-3)) 
					+ (clic_y-( 50+agent.nodeList.get(i).y*8-3)) * (clic_y-( 50+agent.nodeList.get(i).y*8-3));
					if (d<dmin){
						dmin=d;
						agent.observer.setSelectedNode(i);
					}
				}
			}
		}
		
		// compute average translation
		int nbPos=InteractionList.size1*InteractionList.size2*3;
		if (!agent.nodeList.getMesh()){
			for (int l=0;l<6;l++){
				float sum=0;
				float sum2=0;
				
				float averageX=0;
				float averageY=0;
				
				float averageR=0;
				
				if (display[l]){
					
					averageX=0;
					averageY=0;
					
					averageR=0;
					
					for (int i=color; i<InteractionList.size1*InteractionList.size2*3;i+=3){
	
						// for each node, draw links between this node and average position of high weights
						float x=0;
						float y=0;
						float n=0;

						for (int k=0;k<6;k++){
							for (int j=1; j<nbPos;j++){
								if (agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos]>5){	
									x+=agent.nodeList.get(j).x * agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
									y+=agent.nodeList.get(j).y * agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
									n+=agent.signatureList.get(i+l*nbPos).signature[j+k*nbPos];
								}
							}
						}
						
						
						float x0=agent.nodeList.get(i).x;
						float y0=agent.nodeList.get(i).y;
						
						
						if (n>0){
							x=x/n;
							y=y/n;
							
							// compute average translation
							if (l<=1){
								averageX+=x-x0;
								averageY+=y-y0;
								sum++;
							}
							
							// compute average rotation
							x0=x0-3*14.5f;			// center on x axis
							x=x-3*14.5f;
							y0=y0-3*0.5f;
							y=y-3*0.5f;
							averageR+= 	Math.acos(	( (x*x0) + (y*y0) )
										/ (Math.sqrt(x*x+y*y)*Math.sqrt(x0*x0+y0*y0)) );
							
							if (l==2) System.out.println(Math.acos(	( (x*x0) + (y*y0) )
										/ (Math.sqrt(x*x+y*y)*Math.sqrt(x0*x0+y0*y0)) )*180/Math.PI);
							sum2++;
						}
					}
					
					if (sum>1){
						averageX=averageX/sum;
						averageY=averageY/sum;
					}
					if (sum2>1){
						averageR=averageR/sum2;
					}
					System.out.println("+++ "+l+" : nb="+sum+" ; translation={"+(averageX/3)+", "+(averageY/3)+"} ; rotation={"+(averageR*180/Math.PI)+"}");
					
				}
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

}
